/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.dao.hibernate;

import java.util.LinkedHashMap;
import java.util.List;
import java.util.Map;

import org.hibernate.Criteria;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Criterion;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import com.rjconsultores.ventaboletos.dao.ConfigRestriccionPagoDAO;
import com.rjconsultores.ventaboletos.entidad.ConfigRestriccionPago;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.FormaPago;
import com.rjconsultores.ventaboletos.entidad.RestriccionPago;

/**
 *
 * @author Administrador
 */
@Repository("configRestriccionPagoDAO")
public class ConfigRestriccionPagoHibernateDAO
        extends GenericHibernateDAO<ConfigRestriccionPago, Integer>
        implements ConfigRestriccionPagoDAO {

    @Autowired
    public ConfigRestriccionPagoHibernateDAO(@Qualifier("sessionFactory") SessionFactory factory) {
        setSessionFactory(factory);
    }

    @Override
    public List<ConfigRestriccionPago> obtenerTodos() {
        Criteria c = getSession().createCriteria(getPersistentClass());
        c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));

        return c.list();
    }

    public List<ConfigRestriccionPago> buscar(FormaPago formaPago,
            RestriccionPago restriccionPago, Empresa empresa) {
        Criteria c = getSession().createCriteria(getPersistentClass());
        c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));

        c.add(Restrictions.eq("restriccion", restriccionPago));
        c.add(Restrictions.eq("formaPago", formaPago));
        
        Criterion empresaNull = Restrictions.isNull("empresa");
        Criterion empresaMenosUm = Restrictions.eq("empresa", new Empresa(-1));
        
        if(empresa != null) {
        	Criterion empresaIgual = Restrictions.eq("empresa", empresa);
        	c.add(Restrictions.or(Restrictions.or(empresaNull, empresaMenosUm), empresaIgual));
        } else {
        	c.add(Restrictions.or(empresaNull, empresaMenosUm));
        }

        return c.list();
    }

    public Map<Integer, ConfigRestriccionPago> buscar(FormaPago formaPago) {
        Criteria c = getSession().createCriteria(getPersistentClass());
        c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));
        c.add(Restrictions.eq("formaPago", formaPago));

        Map<Integer, ConfigRestriccionPago> mapRespuesta = new LinkedHashMap<Integer, ConfigRestriccionPago>();

        List<ConfigRestriccionPago> list = c.list();
        if (!list.isEmpty()) {
            for (ConfigRestriccionPago crp : list) {
                mapRespuesta.put(crp.getRestriccion().getRestriccionId().intValue(), crp);
            }
        }

        return mapRespuesta;
    }
}
