package com.rjconsultores.ventaboletos.dao.hibernate;

import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.HibernateException;
import org.hibernate.Session;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import com.rjconsultores.ventaboletos.dao.OCDParamDAO;
import com.rjconsultores.ventaboletos.entidad.AbastoCentral;
import com.rjconsultores.ventaboletos.entidad.ControleEstoqueMigracao;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.OCDParam;

@Repository("ocdParamDAO")
public class OCDParamHibernateDAO extends GenericHibernateDAO<OCDParam, Integer>  implements OCDParamDAO {
	
	@Autowired
	public OCDParamHibernateDAO(
			@Qualifier("sessionFactory") SessionFactory factory) {
		setSessionFactory(factory);
	}

	@Override
	public List<OCDParam> obtenerTodos() {
		List<OCDParam> params = findByCriteria(Restrictions.eq(ACTIVO, Boolean.TRUE));
		return params;
	}

	@Override
	public OCDParam buscaOCDParamPorEmpresa(Empresa empresa) {
		
		Criteria c = makeCriteria();
		c.add(Restrictions.eq("empresa", empresa));
		c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));
		
		return (OCDParam)c.uniqueResult();
	}

	@Override
	public List<OCDParam> buscaOCDParams(Empresa empresa) {				
		List<OCDParam> params = findByCriteria(Restrictions.eq("empresa", empresa), Restrictions.eq(ACTIVO, Boolean.TRUE));		
		return params;
	}

	@Override
	public OCDParam suscribirActualizar(OCDParam entidad){
		try {
			if (entidad.getOcdparamId() != null){
				this.getHibernateTemplate().merge(entidad);
			} else {
				this.getHibernateTemplate().save(entidad);
			}
			return entidad;
		} catch (final HibernateException ex) {
			throw convertHibernateAccessException(ex);
		}
	}

}
