package com.rjconsultores.ventaboletos.dao.hibernate;

import java.util.ArrayList;
import java.util.List;

import org.hibernate.Criteria;
import org.hibernate.SessionFactory;
import org.hibernate.criterion.Restrictions;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.beans.factory.annotation.Qualifier;
import org.springframework.stereotype.Repository;

import com.rjconsultores.ventaboletos.dao.TarifaGrupoContratoDAO;
import com.rjconsultores.ventaboletos.entidad.TarifaGrupoContrato;
import com.rjconsultores.ventaboletos.vo.configuracioneccomerciales.TarifaGrupoContratoVO;

@SuppressWarnings("unchecked")
@Repository("tarifaGrupoContratoDAO")
public class TarifaGrupoContratoHibernateDAO extends GenericHibernateDAO<TarifaGrupoContrato, Integer> implements TarifaGrupoContratoDAO {

    @Autowired
    public TarifaGrupoContratoHibernateDAO(@Qualifier("sessionFactory") SessionFactory factory) {
        setSessionFactory(factory);
    }

	@Override
    public List<TarifaGrupoContrato> obtenerTodos() {
        Criteria c = getSession().createCriteria(getPersistentClass());
        c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));

        return c.list();
    }

	@Override
	public boolean gravarTarifasGrupoContrato(List<TarifaGrupoContrato> tarifas) {

		for (TarifaGrupoContrato tarifa : tarifas) {
			if(tarifa.getTarifaGrupoContratoId() == null) {
				suscribir(tarifa);
			}else {
				actualizacion(tarifa);
			}
		}

		return true;
	}

	@Override
	public TarifaGrupoContrato obterTarifaPeloVo(TarifaGrupoContratoVO tarifa) {
		if( tarifa.getTarifaGrupoContratoId() != null ) {
			return obtenerID(tarifa.getTarifaGrupoContratoId());
		}
		return null;
	}

	@Override
	public List<TarifaGrupoContratoVO> buscarTarifasGrupo(Long grupoContratoId) {
		List<TarifaGrupoContratoVO> retorno = new ArrayList<TarifaGrupoContratoVO>();
		
		List<TarifaGrupoContrato> tarifas = buscarTarifaPeloGrupo(grupoContratoId);
		for (TarifaGrupoContrato tarifa : tarifas) {
			TarifaGrupoContratoVO tarifaVO = new TarifaGrupoContratoVO();
			tarifaVO.setTarifaGrupoContratoId( tarifa.getTarifaGrupoContratoId().toString() );
			tarifaVO.setGrupoContratoId( tarifa.getGrupoContrato().getGrupoContratoId().toString());
			tarifaVO.setCveOrigem(tarifa.getOrigem().getCveparada());
			tarifaVO.setDescOrigem(tarifa.getOrigem().getDescparada());
			tarifaVO.setCveDestino(tarifa.getDestino().getCveparada());
			tarifaVO.setDescDestino(tarifa.getDestino().getDescparada());
			tarifaVO.setTarifa(tarifa.getTarifa().toString());
			retorno.add(tarifaVO);
		}
		
		return retorno;
	}

	@Override
	public List<TarifaGrupoContrato> buscarTarifaPeloGrupo(Long grupo) {
		Criteria c = getSession().createCriteria(getPersistentClass());
		c.add(Restrictions.eq("grupoContrato.grupoContratoId", grupo));
        c.add(Restrictions.eq(ACTIVO, Boolean.TRUE));

        return c.list();
	}
}
