package com.rjconsultores.ventaboletos.entidad;

import java.io.Serializable;
import java.util.Date;

import javax.persistence.Basic;
import javax.persistence.Column;
import javax.persistence.Entity;
import javax.persistence.GeneratedValue;
import javax.persistence.GenerationType;
import javax.persistence.Id;
import javax.persistence.JoinColumn;
import javax.persistence.ManyToOne;
import javax.persistence.SequenceGenerator;
import javax.persistence.Table;
import javax.persistence.Temporal;
import javax.persistence.TemporalType;
import javax.persistence.Transient;

import org.hibernate.Hibernate;

import br.com.rjconsultores.auditador.annotations.AuditarClasse;
import br.com.rjconsultores.auditador.annotations.NaoAuditar;
import br.com.rjconsultores.auditador.interfaces.Auditavel;

@Entity
@AuditarClasse(nome = "CategoriaRutaExcecao", tela = "auditarClasse.CategoriaRutaExcecao")
@SequenceGenerator(name = "CATEGORIA_RUTA_EXCECAO_SEQ", sequenceName = "CATEGORIA_RUTA_EXCECAO_SEQ", allocationSize = 1)
@Table(name = "CATEGORIA_RUTA_EXCECAO")
public class CategoriaRutaExcecao implements Serializable, Auditavel<CategoriaRutaExcecao> {

    private static final long serialVersionUID = 1L;
    @Id
    @Basic(optional = false)
    @GeneratedValue(strategy = GenerationType.AUTO, generator = "CATEGORIA_RUTA_EXCECAO_SEQ")
    @Column(name = "CATEGORIARUTAEXCECAO_ID")
    private Integer categoriaRutaExcecaoId;
    @Column(name = "ACTIVO")
    private Boolean activo;
    @Column(name = "FECMODIF")
    @Temporal(TemporalType.TIMESTAMP)
    private Date fecmodif;
    @Column(name = "USUARIO_ID")
    private Integer usuarioId;
    @JoinColumn(name = "RUTA_ID", referencedColumnName = "RUTA_ID")
    @ManyToOne
    private Ruta ruta;
    @JoinColumn(name = "CATEGORIACTRL_ID", referencedColumnName = "CATEGORIACTRL_ID")
    @ManyToOne
    private CategoriaCtrl categoriaCtrl;
	@Transient
	@NaoAuditar
	private CategoriaRutaExcecao categoriaRutaExcecaoClone;

    public CategoriaRutaExcecao() {
    }

    public CategoriaRutaExcecao(Integer categoriaRutaExcecaoId) {
        this.categoriaRutaExcecaoId = categoriaRutaExcecaoId;
    }

    public Integer getCategoriaRutaExcecaoId() {
		return categoriaRutaExcecaoId;
	}

	public void setCategoriaRutaExcecaoId(Integer categoriaRutaExcecaoId) {
		this.categoriaRutaExcecaoId = categoriaRutaExcecaoId;
	}

	public Boolean getActivo() {
        return activo;
    }

    public void setActivo(Boolean activo) {
        this.activo = activo;
    }

    public Date getFecmodif() {
        return fecmodif;
    }

    public void setFecmodif(Date fecmodif) {
        this.fecmodif = fecmodif;
    }

    public Integer getUsuarioId() {
        return usuarioId;
    }

    public void setUsuarioId(Integer usuarioId) {
        this.usuarioId = usuarioId;
    }

   

    public Ruta getRuta() {
		return ruta;
	}

	public void setRuta(Ruta ruta) {
		this.ruta = ruta;
	}

	public CategoriaCtrl getCategoriaCtrl() {
        return categoriaCtrl;
    }

    public void setCategoriaCtrl(CategoriaCtrl categoriaCtrl) {
        this.categoriaCtrl = categoriaCtrl;
    }

    @Override
    public boolean equals(Object obj) {
        if (obj == null) {
            return false;
        }
        if (getClass() != obj.getClass()) {
            return false;
        }
        final CategoriaRutaExcecao other = (CategoriaRutaExcecao) obj;
        if (this.ruta != other.ruta && (this.ruta == null || !this.ruta.equals(other.ruta))) {
            return false;
        }
        if (this.categoriaCtrl != other.categoriaCtrl && (this.categoriaCtrl == null || !this.categoriaCtrl.equals(other.categoriaCtrl))) {
            return false;
        }
        return true;
    }

    @Override
    public int hashCode() {
        int hash = 7;
        hash = 97 * hash + (this.ruta != null ? this.ruta.hashCode() : 0);
        hash = 97 * hash + (this.categoriaCtrl != null ? this.categoriaCtrl.hashCode() : 0);
        return hash;
    }

    @Override
    public String toString() {
        return "com.rjconsultores.ventaboletos.entidad.CategoriaRutaExcecao[categoriaRutaExcecaoId=" + categoriaRutaExcecaoId + "]";
    }
    
	@Override
	public void clonar() throws CloneNotSupportedException {
		categoriaRutaExcecaoClone = new CategoriaRutaExcecao();
		categoriaRutaExcecaoClone = (CategoriaRutaExcecao) this.clone();
		Hibernate.initialize(categoriaRutaExcecaoClone.getCategoriaCtrl());

	}

	@Override
	public CategoriaRutaExcecao getCloneObject() throws CloneNotSupportedException {
		return categoriaRutaExcecaoClone;
	}

	@Override
	public String getTextoInclusaoExclusao() {
		return String.format("ID [%s]", getRuta().getNumRuta());
	}

}
