/*
 * To change this template, choose Tools | Templates
 * and open the template in the editor.
 */
package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.CiudadDAO;
import com.rjconsultores.ventaboletos.entidad.Ciudad;
import com.rjconsultores.ventaboletos.entidad.Estado;
import com.rjconsultores.ventaboletos.entidad.Plaza;
import com.rjconsultores.ventaboletos.service.CiudadService;
import com.rjconsultores.ventaboletos.service.PlazaService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

/**
 * 
 * @author MCosso
 */
@Service("ciudadService")
public class CiudadServiceImpl implements CiudadService {

	private static final Logger log = LogManager.getLogger(CiudadServiceImpl.class);

	@Autowired
	private CiudadDAO ciudadDAO;
	@Autowired
	private PlazaService plazaService;

	public List<Ciudad> obtenerTodos() {
		return ciudadDAO.obtenerTodos();
	}

	public Ciudad obtenerID(Integer id) {
		return ciudadDAO.obtenerID(id);
	}

	@Transactional
	public Ciudad suscribir(Ciudad entidad) {

		if (entidad.getPlaza() == null) {
			entidad.setPlaza(getPlazaCiudad());
		}

		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return ciudadDAO.suscribir(entidad);
	}

	@Transactional
	public Ciudad actualizacion(Ciudad entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return ciudadDAO.actualizacion(entidad);
	}

	@Transactional
	public void borrar(Ciudad entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		ciudadDAO.actualizacion(entidad);
	}

	private Plaza getPlazaCiudad() {
		try {
			return plazaService.obtenerTodos().iterator().next();
		} catch (Exception e) {
			log.error("", e);
		}
		return null;
	}

	public List<Ciudad> buscar(String nombciudad, Estado estado, Plaza plaza) {
		return ciudadDAO.buscar(nombciudad, estado, plaza);
	}

	public List<Ciudad> buscaLike(String strCiudad) {
		return ciudadDAO.buscaLike(strCiudad);
	}

	@Override
	public List<Ciudad> buscarPorEstado(Estado estado) {
		return ciudadDAO.buscarPorEstado(estado);
	}
	
	@Override
	public List<Ciudad> buscaCodMun(Integer codMun, Estado estado) {
		return ciudadDAO.buscaCodMun(codMun, estado);
	}
}
