package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.ContratoDAO;
import com.rjconsultores.ventaboletos.dao.DescontoContratoDAO;
import com.rjconsultores.ventaboletos.entidad.ContratoCorporativo;
import com.rjconsultores.ventaboletos.entidad.DescontoContrato;
import com.rjconsultores.ventaboletos.service.ContratoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("contratoService")
public class ContratoServiceImpl implements ContratoService {

	@Autowired
	private ContratoDAO contratoDAO;
	
	@Autowired
	private DescontoContratoDAO descontoContratoDAO;

	public List<ContratoCorporativo> obtenerTodos() {
		return contratoDAO.obtenerTodos();
	}

	public ContratoCorporativo obtenerID(Long id) {
		return contratoDAO.obtenerID(id);
	}

	@Transactional
	public ContratoCorporativo suscribirActualizar(ContratoCorporativo entidad) {		
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecModif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		if (entidad.getContratoId() == null){
			entidad.setStatusContrato(1);
			return contratoDAO.suscribir(entidad);
		}else{
			return contratoDAO.actualizacion(entidad);
		}
	}

	@Transactional
	public void borrar(ContratoCorporativo entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecModif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		contratoDAO.actualizacion(entidad);
	}

	@Override
	public List<ContratoCorporativo> buscaLike(String strContratoCorporativo) {
		return contratoDAO.buscaLike( strContratoCorporativo);
	}

	@Override
	public boolean existe(ContratoCorporativo contrato) {
		return contratoDAO.existe(contrato);
	}

	@Override
	@Transactional
	public void gravarDescontosContrato(List<DescontoContrato> lsDescontoBanco) {
		descontoContratoDAO.gravarDescontosContrato(lsDescontoBanco);		
	}
}
