package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.EsquemaCorridaInfoDAO;
import com.rjconsultores.ventaboletos.entidad.EsquemaCorrida;
import com.rjconsultores.ventaboletos.entidad.EsquemaCorridaInfo;
import com.rjconsultores.ventaboletos.service.EsquemaCorridaInfoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("esquemaCorridaInfoService")
public class EsquemaCorridaInfoServiceImpl implements EsquemaCorridaInfoService {

	@Autowired
	private EsquemaCorridaInfoDAO esquemaCorridaInfoDAO;

	@Override
	public List<EsquemaCorridaInfo> obtenerTodos() {
		return esquemaCorridaInfoDAO.obtenerTodos();
	}

	@Override
	public EsquemaCorridaInfo obtenerID(Integer id) {
		return esquemaCorridaInfoDAO.obtenerID(id);
	}

	@Transactional
	@Override
	public  EsquemaCorridaInfo suscribir( EsquemaCorridaInfo entidad) {
		
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);
	
		return esquemaCorridaInfoDAO.suscribir(entidad);
	}

	@Transactional
	@Override
	public EsquemaCorridaInfo actualizacion(EsquemaCorridaInfo entidad) {
		
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);
		return esquemaCorridaInfoDAO.actualizacion(entidad);
	}

	@Transactional
	@Override
	public void borrar(EsquemaCorridaInfo entidad) {
		
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);
		esquemaCorridaInfoDAO.actualizacion(entidad);
	}

	@Override
	public List<EsquemaCorridaInfo> obtenerPorEsquemaCorrida(EsquemaCorrida esquemaCorrida) {
		return esquemaCorridaInfoDAO.obtenerPorEsquemaCorrida(esquemaCorrida);
	}

}
