package com.rjconsultores.ventaboletos.service.impl;

import java.sql.Date;
import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.FeriadoDAO;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Estado;
import com.rjconsultores.ventaboletos.entidad.Feriado;
import com.rjconsultores.ventaboletos.service.FeriadoService;
import com.rjconsultores.ventaboletos.utilerias.FeriadoCache;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("feriadoService")
public class FeriadoServiceImpl implements FeriadoService {

	@Autowired
	private FeriadoDAO feriadoDAO;
	
	@Autowired
	private FeriadoCache feriadoCache;

	public List<Feriado> obtenerTodos() {
		return feriadoDAO.obtenerTodos();
	}

	public Feriado obtenerID(Integer id) {
		return feriadoDAO.obtenerID(id);
	}

	@Transactional
	public Feriado suscribir(Feriado entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);
		
		feriadoCache.limparData(entidad.getFecferiado(),  entidad.getEmpresa() == null ? null : entidad.getEmpresa().getEmpresaId(), entidad.getEstado() == null ? null : entidad.getEstado().getEstadoId());

		return feriadoDAO.suscribir(entidad);
	}

	@Transactional
	public Feriado actualizacion(Feriado entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);
		
		feriadoCache.limparData(entidad.getFecferiado(),  entidad.getEmpresa() == null ? null : entidad.getEmpresa().getEmpresaId(), entidad.getEstado() == null ? null : entidad.getEstado().getEstadoId());

		return feriadoDAO.actualizacion(entidad);
	}

	@Transactional
	public void borrar(Feriado entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);
		
		feriadoCache.limparData(entidad.getFecferiado(),  entidad.getEmpresa() == null ? null : entidad.getEmpresa().getEmpresaId(), entidad.getEstado() == null ? null : entidad.getEstado().getEstadoId());

		feriadoDAO.actualizacion(entidad);
	}

	public List<Feriado> buscar(Date fecferiado) {
		return feriadoDAO.buscar(fecferiado);
	}

	@Override
	public List<Feriado> buscarDataEmpresaEstado(Date data, Empresa empresa, Estado estado) {
		return feriadoDAO.buscarDataEmpresaEstado(data, empresa, estado);
	}
}
