package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.FolioPreimpresoDAO;
import com.rjconsultores.ventaboletos.entidad.Empresa;
import com.rjconsultores.ventaboletos.entidad.Estacion;
import com.rjconsultores.ventaboletos.entidad.FolioPreimpreso;
import com.rjconsultores.ventaboletos.service.FolioPreimpresoService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("folioPreimpresoService")
public class FolioPreimpresoServiceImpl implements FolioPreimpresoService {

	@Autowired
	private FolioPreimpresoDAO folioPreimpresoDAO;

	@Override
	public List<FolioPreimpreso> obtenerTodos() {
		return folioPreimpresoDAO.obtenerTodos();
	}

	@Override
	public FolioPreimpreso obtenerID(Integer id) {
		return folioPreimpresoDAO.obtenerID(id);
	}

	@Transactional
	public FolioPreimpreso suscribir(FolioPreimpreso entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return folioPreimpresoDAO.suscribir(entidad);
	}

	@Transactional
	public FolioPreimpreso actualizacion(FolioPreimpreso entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);

		return folioPreimpresoDAO.actualizacion(entidad);
	}

	@Transactional
	public void borrar(FolioPreimpreso entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		folioPreimpresoDAO.actualizacion(entidad);
	}

	@Override
	public FolioPreimpreso buscaFolioPreImpressoEstacionImpresora(Estacion estacion, Empresa empresa) {
		return folioPreimpresoDAO.buscaFolioPreImpressoEstacionImpresora(estacion, empresa);
	}

}
