package com.rjconsultores.ventaboletos.service.impl;

import java.util.Calendar;
import java.util.List;

import org.apache.logging.log4j.LogManager;
import org.apache.logging.log4j.Logger;
import org.springframework.beans.factory.annotation.Autowired;
import org.springframework.stereotype.Service;
import org.springframework.transaction.annotation.Transactional;

import com.rjconsultores.ventaboletos.dao.OrgaoConcedenteDAO;
import com.rjconsultores.ventaboletos.entidad.OrgaoConcedente;
import com.rjconsultores.ventaboletos.service.LogAuditoriaService;
import com.rjconsultores.ventaboletos.service.OrgaoConcedenteService;
import com.rjconsultores.ventaboletos.utilerias.UsuarioLogado;

@Service("orgaoConcedenteService")
public class OrgaoConcedenteServiceImpl implements OrgaoConcedenteService {

	@Autowired
	private OrgaoConcedenteDAO orgaoConcedenteDAO;
	@Autowired
	private LogAuditoriaService logAuditoriaService;
	private static final Logger log = LogManager.getLogger(OrgaoConcedenteServiceImpl.class);

	public List<OrgaoConcedente> obtenerTodos() {
		return orgaoConcedenteDAO.obtenerTodos();
	}

	public OrgaoConcedente obtenerID(Integer id) {
		
		OrgaoConcedente orgaoConcedente = orgaoConcedenteDAO.obtenerID(id);
		try {
			orgaoConcedente.clonar();
		} catch (Exception e) {
			log.error(e.getMessage(), e);
		}
		return orgaoConcedente;
		
	}

	@Transactional
	public OrgaoConcedente suscribir(OrgaoConcedente entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.TRUE);
		
		orgaoConcedenteDAO.suscribir(entidad);
		logAuditoriaService.auditar(null, entidad, null);
		
		return entidad;
	}

	@Transactional
	public OrgaoConcedente actualizacion(OrgaoConcedente entidad) {
		
		try {
			OrgaoConcedente originalClone = entidad.getCloneObject();
			
			entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
			entidad.setFecmodif(Calendar.getInstance().getTime());
			entidad.setActivo(Boolean.TRUE);

			entidad =  orgaoConcedenteDAO.actualizacion(entidad);
			logAuditoriaService.auditar(originalClone, entidad, null);
		} catch (Exception e) {
			log.error(e);
		}
	
	 return entidad;
	}

	@Transactional
	public void borrar(OrgaoConcedente entidad) {
		entidad.setUsuarioId(UsuarioLogado.getUsuarioLogado().getUsuarioId());
		entidad.setFecmodif(Calendar.getInstance().getTime());
		entidad.setActivo(Boolean.FALSE);

		orgaoConcedenteDAO.actualizacion(entidad);
		logAuditoriaService.auditarExclusao(entidad, null);

	}

	public List<OrgaoConcedente> buscar(String desc) {
		return orgaoConcedenteDAO.buscar(desc);
	}

	@Override
	public List<OrgaoConcedente> obtenerTodosExceto(Integer... id) {
		return orgaoConcedenteDAO.obtenerTodosExceto(id);
	}
}
